#!/usr/bin/env python

"""
photos_n800.py - A photo browser demonstration.
This file is part of the PyQt for Desktop and Embedded Devices package.

Copyright (C) 2009 David Boddie <david.boddie@nokia.com>
Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
All rights reserved.

Contact: Nokia Corporation (qt-info@nokia.com)

You may use this file under the terms of the BSD license as follows:

"Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor the names
   of its contributors may be used to endorse or promote products derived from
   this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
"""

import sys
from PyQt4.QtCore import *
from PyQt4.QtGui import *
import extra_photos_rc

Width = 600
Height = 400
Photo_Width = 400
Photo_Height = 300

class Window(QWidget):

    def __init__(self, photos, parent = None):
    
        QWidget.__init__(self, parent)
        self.setAutoFillBackground(True)
        
        palette = QPalette()
        gradient = QLinearGradient(0, 0, 0, Height)
        gradient.setColorAt(0, QColor(0x2e, 0x5e, 0x3e))
        gradient.setColorAt(0.65, QColor(16, 16, 16))
        gradient.setColorAt(1, QColor(127, 127, 127))
        palette.setBrush(QPalette.Window, gradient)
        self.setPalette(palette)
        
        self.pressed_x = 0
        self.distance = 16
        self.direction = 1
        self.offset = 0
        self.photo = 0
        self.photos = photos
        self.photo_x = 0
        
        self.processPhotos()
        
        self.timeline = QTimeLine(1000)
        self.timeline.setFrameRange(0, Width)
        self.connect(self.timeline, SIGNAL("frameChanged(int)"), self.animate)
    
    def animate(self, value):
    
        new_offset = value
        change = new_offset - self.offset
        self.scroll(-self.direction * change, 0)
        self.offset = new_offset
        self.photo_x += -self.direction * change
    
    def mousePressEvent(self, event):
    
        self.pressed_x = event.x()
	event.accept()
    
    def mouseReleaseEvent(self, event):
    
        if event.x() > self.pressed_x + self.distance:
        
            event.accept()
            if self.timeline.state() == QTimeLine.Running:
            
                if self.direction == 1 and \
                    self.timeline.direction() == QTimeLine.Forward:
                
                    self.timeline.setDirection(QTimeLine.Backward)
                    self.photo = (self.photo - 1) % len(self.photos)
                
                elif self.direction == -1 and \
                    self.timeline.direction() == QTimeLine.Backward:
                
                    self.timeline.setDirection(QTimeLine.Forward)
                    self.photo = (self.photo - 1) % len(self.photos)
            
            else:
            
                self.timeline.setDirection(QTimeLine.Forward)
                self.direction = -1
                self.offset = 0
                self.photo_x = -Width
                self.photo = (self.photo - 1) % len(self.photos)
                self.timeline.start()
        
        elif event.x() < self.pressed_x - self.distance:
        
            event.accept()
            if self.timeline.state() == QTimeLine.Running:
            
                if self.direction == -1 and \
                    self.timeline.direction() == QTimeLine.Forward:
                
                    self.timeline.setDirection(QTimeLine.Backward)
                    self.photo = (self.photo + 1) % len(self.photos)
                
                elif self.direction == 1 and \
                    self.timeline.direction() == QTimeLine.Backward:
                
                    self.timeline.setDirection(QTimeLine.Forward)
                    self.photo = (self.photo + 1) % len(self.photos)
            
            else:
            
                self.timeline.setDirection(QTimeLine.Forward)
                self.direction = 1
                self.offset = 0
                self.photo_x = Width
                self.photo = (self.photo + 1) % len(self.photos)
                self.timeline.start()
        
        elif event.key() == Qt.Key_Q:
        
            qApp.quit()
        
        else:
            event.ignore()
    
    def paintEvent(self, event):
    
        painter = QPainter()
        painter.begin(self)
        dx, dy, pixmap = self.photos[self.photo]
        reflection = self.reflections[self.photo]
        
        if self.timeline.direction() == QTimeLine.Forward:
            pass
        elif self.direction == -1:
            # When reversing left scrolling, draw the image returning from
            # the right.
            dx += Width
        else:
            # When reversing right scrolling, draw the image returning from
            # the left.
            dx -= Width
        
        painter.drawPixmap(self.photo_x + dx + (Width - Photo_Width)/2, 10 + dy, pixmap)
        painter.drawPixmap(self.photo_x + dx + (Width - Photo_Width)/2, Photo_Height + 10, reflection)
        painter.end()
    
    def minimumSizeHint(self):
        return QSize(Width, Height)
    
    def processPhotos(self):
    
        self.reflections = []
        for dx, dy, photo in self.photos:
        
            reflected_pixmap = photo.transformed(QMatrix(1, 0, 0, -1, 0, 0))
            reflected_pixmap = reflected_pixmap.copy(
                0, 0, reflected_pixmap.width(), min(reflected_pixmap.height(), Height - Photo_Height)
                )
            alpha = QPixmap(reflected_pixmap.size())
            alpha.fill(QColor(40, 40, 40))
            reflected_pixmap.setAlphaChannel(alpha)
            self.reflections.append(reflected_pixmap)


names = (":/images/greenphone-python.jpg",
         ":/images/dsc02120-Il-Duomo.jpg", ":/images/dsc02133-Hotel-Astoria.jpg")

def load_photos():

    photos = []
    for name in names:
        pixmap = QPixmap()
        if pixmap.load(name):
            if pixmap.width() > Photo_Width or pixmap.height() > Photo_Height:
                pixmap = pixmap.scaled(Photo_Width, Photo_Height, Qt.KeepAspectRatio, Qt.SmoothTransformation)
            photos.append((Photo_Width/2 - pixmap.width()/2, Photo_Height - pixmap.height(), pixmap))
    
    return photos


if __name__ == "__main__":

    app = QApplication(sys.argv)
    photos = load_photos()
    
    if "-f" not in sys.argv:
        frame = QWidget()
        layout = QHBoxLayout(frame)
        window = Window(photos)
	window.setFixedSize(Width, Height)
        layout.addWidget(window)
        frame.show()
    else:
        window = Window(photos)
        window.showFullScreen()
    sys.exit(app.exec_())
